/**********************************************************************//**
***************************************************************************
*** @file    list.h
***
*** @brief   Pointer Lists
***
*** @todo    Author information
***
***
*** @revisionHistory
***
***     Date          Version    Author  Changes
***     10.06.2013    0.1        MBE     Initial Version from Poppo.
***     Dec 30, 2013  3.13       ME      PtrList_addPtr() now returns error code
***
*** @endRevisionHistory
***************************************************************************
***************************************************************************/
#ifndef __LIST_H
#define __LIST_H
/*--GROUP-----------------------------------------------------------------*/
/**
* @cond ( vclib || extlib )
* @defgroup pointer_list  Pointer Lists (SPtrList)
* 
*  @brief Organize Sets of Pointers to the same Type of Data.
* 
*    The routines in this section handle a list of pointers (stored as an array of pointers)
*    for various applications.
*    Memory for the pointer list is allocated and released automatically.
*    It is also possible to consider the pointers in the list as pointers to
*    a memory block which should be released when the pointer is removed from the
*    list or the list is erased as a whole.
*    
*   A pointer list has the following Behaviour:
*   - An empty pointer list contains no elements, the @p size counter is @c 0.
*   - Adding a pointer increments the @p size counter by one.
*   - The functions for the pointer lists contain automatic allocation for pointer memory.
*     A small pointer list requires @ref MIN_ALLOC words. If the space is not sufficient
*     twice the amount of the previously allocated pointer memory will be allocated.
*   - As soon as the list becomes empty memory space for the pointers will be released.
*   - One can set the @a do_release_pointers flag to @c 1 for comfortable
*     deallocation of the memory to which the pointers point to. 
* 
*   Usage is as follows:
*   1. Every @ref SPtrList must be first set to the @ref NULL_LIST.
*   2. Decide if  memory pointed to by the pointers in the list should be kept
*      or released when the pointer or the whole list is removed. Call @ref PtrList_setReleasePointers()
*      to set the appropriate mode.
*   3. Do -- for example -- some of the following tasks:
*      - Check if the @ref SPtrList contains no Elements by calling @ref isNullList().
*      - Add/Remove/Delete Pointers to the @ref SPtrList by calling the 
*        appropriate functions @ref PtrList_addPtr()/@ref PtrList_removePtr()/@ref PtrList_deleteItem().
*      - Sort @ref SPtrList by calling @ref PtrList_sort().
*      - Get First/Index/Last Pointer of the List by calling @ref PtrList_getFirstPtr()/@ref PtrList_getPtr()/@ref PtrList_getLastPtr().
* 
*    
* @{
*/

#define MIN_ALLOC   	 (1024)  /* this is the minimum number of units to be allocated by the system    */
#define DO_RELEASE_POINTERS	(1)

#ifndef _MAX_
    #define _MAX_(x,y) ( (x) > (y) ? (x) : ( y) )
#endif

#ifdef __cplusplus
extern "C" {
#endif
    
  /*--*STRUCT*--------------------------------------------------------------*/
  /**
  *  @brief  Pointer List Data Holder.
  *
  *    This structure manages pointer lists. 
  */
  typedef struct _SPtrListStruct
  {
    void          **data;              /*!<  List of Pointers.                        */
    int           reserved;            /*!<  Only for Internal Use.                   */
    volatile int  size;                /*!<  Count of Pointers stored in List.        */
    int           do_release_pointers; /*!<  Deallocates Pointers Automatically If Set (See Text). */
  } SPtrList;
  
  // //To be the same as NULL_LIST
  // #define               NULL_SPTRLIST  {NULL, 0, 0, 0}
  extern const SPtrList NULL_LIST;

  extern void  PtrList_setReleasePointers(SPtrList *ptr_list, int release_pointers);
  extern int   PtrList_isNullList(const SPtrList* ptr_list);
  extern void* PtrList_getPtr(const SPtrList* ptr_list, int index);
  extern void* PtrList_getLastPtr(const SPtrList* ptr_list);
  extern void* PtrList_getFirstPtr(const SPtrList* ptr_list);
  extern int   PtrList_addPtr(SPtrList* ptr_list, void* ptr);
  extern void  PtrList_removePtr(SPtrList* ptr_list, int index);
  extern void  PtrList_deleteItem(SPtrList* ptr_list, int index);
  extern void  PtrList_updateList(SPtrList* ptr_list);
  extern void  PtrList_clearList(SPtrList* ptr_list);
  extern void  *PtrList_realloc(void *ptr, int size);
  extern void  PtrList_copy(SPtrList* ptr_list_src, SPtrList* ptr_list_dst);
  extern void  PtrList_reverse(SPtrList* ptr_list);
  extern void  PtrList_sort(SPtrList *ptr_list, int offset);
  extern void  PtrList_bubblesort(SPtrList *ptr_list, int offset);
  extern void  PtrList_shellsort(SPtrList *ptr_list, int offset);
  extern void  PtrList_heapsort(SPtrList *ptr_list, int offset);
         void  PtrList_print(SPtrList* pLst);
         I32   PtrList_split(SPtrList* pLstToSplit, SPtrList* pLstMinor, SPtrList* pLstMajor, I32 i32MajorFirstIdx);
         I32   PtrListPtr_move(SPtrList* pLstFrom, SPtrList* pLstTo, I32 i32Idx);
         void  PtrList_shuffle(const SPtrList* ptr_list, I32 SwapCount);

#ifdef __cplusplus
}
#endif


/**@} @endcond*/
#endif /* __LIST_H */

/* EOF */
